// Author: Hezhk3

// see https://gl-transitions.com/editor/BowTieHorizontal

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

#define progress fract(iTime)
#define getFromColor(coord) texture2D(inputImageTexture, coord)
#define getToColor(coord) texture2D(inputImageTexture2, coord)

// Author: huynx
// License: MIT

highp float check(highp vec2 p1, highp vec2 p2, highp vec2 p3)
{
    return (p1.x - p3.x) * (p2.y - p3.y) - (p2.x - p3.x) * (p1.y - p3.y);
}

bool PointInTriangle (highp vec2 pt, highp vec2 p1, highp vec2 p2, highp vec2 p3)
{
    bool b1, b2, b3;
    b1 = check(pt, p1, p2) < 0.0;
    b2 = check(pt, p2, p3) < 0.0;
    b3 = check(pt, p3, p1) < 0.0;
    return ((b1 == b2) && (b2 == b3));
}

bool in_left_triangle(highp vec2 p){
    highp vec2 vertex1, vertex2, vertex3;
    vertex1 = vec2(progress, 0.5);
    vertex2 = vec2(0.0, 0.5-progress);
    vertex3 = vec2(0.0, 0.5+progress);
    if (PointInTriangle(p, vertex1, vertex2, vertex3))
    {
        return true;
    }
    return false;
}

bool in_right_triangle(highp vec2 p){
    highp vec2 vertex1, vertex2, vertex3;
    vertex1 = vec2(1.0-progress, 0.5);
    vertex2 = vec2(1.0, 0.5-progress);
    vertex3 = vec2(1.0, 0.5+progress);
    if (PointInTriangle(p, vertex1, vertex2, vertex3))
    {
        return true;
    }
    return false;
}

highp float blur_edge(highp vec2 bot1, highp vec2 bot2, highp vec2 top, highp vec2 testPt)
{
    highp vec2 lineDir = bot1 - top;
    highp vec2 perpDir = vec2(lineDir.y, -lineDir.x);
    highp vec2 dirToPt1 = bot1 - testPt;
    highp float dist1 = abs(dot(normalize(perpDir), dirToPt1));
    
    lineDir = bot2 - top;
    perpDir = vec2(lineDir.y, -lineDir.x);
    dirToPt1 = bot2 - testPt;
    highp float min_dist = min(abs(dot(normalize(perpDir), dirToPt1)), dist1);
    
    if (min_dist < 0.005) {
        return min_dist / 0.005;
    }
    else  {
        return 1.0;
    };
}


highp vec4 transition (highp vec2 uv) {
    if (in_left_triangle(uv))
    {
        if (progress < 0.1)
        {
            return getFromColor(uv);
        }
        if (uv.x < 0.5)
        {
            highp vec2 vertex1 = vec2(progress, 0.5);
            highp vec2 vertex2 = vec2(0.0, 0.5-progress);
            highp vec2 vertex3 = vec2(0.0, 0.5+progress);
            return mix(
                       getFromColor(uv),
                       getToColor(uv),
                       blur_edge(vertex2, vertex3, vertex1, uv)
                       );
        }
        else
        {
            if (progress > 0.0)
            {
                return getToColor(uv);
            }
            else
            {
                return getFromColor(uv);
            }
        }
    }
    else if (in_right_triangle(uv))
    {
        if (uv.x >= 0.5)
        {
            highp vec2 vertex1 = vec2(1.0-progress, 0.5);
            highp vec2 vertex2 = vec2(1.0, 0.5-progress);
            highp vec2 vertex3 = vec2(1.0, 0.5+progress);
            return mix(
                       getFromColor(uv),
                       getToColor(uv),
                       blur_edge(vertex2, vertex3, vertex1, uv)
                       );
        }
        else
        {
            return getFromColor(uv);
        }
    }
    else {
        return getFromColor(uv);
    }
}

void main()
{
    gl_FragColor = transition(textureCoordinate);
}
