// Author: Hezhk3

// see https://gl-transitions.com/editor/cube

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

#define progress fract(iTime)
//#define getFromColor(coord) texture2D(inputImageTexture, coord)
//#define getToColor(coord) texture2D(inputImageTexture2, coord)
#define ratio (iResolution.x / iResolution.y)

#define R_VEC2(p) vec2(p.x, 1.0 - p.y)
#define getFromColor(coord) texture2D(inputImageTexture, R_VEC2(coord))
#define getToColor(coord) texture2D(inputImageTexture2, R_VEC2(coord))

const highp float persp = 0.7; // = 0.7
const highp float unzoom = 0.3; // = 0.3
const highp float reflection = 0.4; // = 0.4
const highp float floating = 3.0; // = 3.0

highp vec2 project (highp vec2 p) {
    return p * vec2(1.0, -1.2) + vec2(0.0, -floating/100.);
}

bool inBounds (highp vec2 p) {
    return all(lessThan(vec2(0.0), p)) && all(lessThan(p, vec2(1.0)));
}

highp vec4 bgColor (highp vec2 p, highp vec2 pfr, highp vec2 pto) {
    highp vec4 c = vec4(0.0, 0.0, 0.0, 1.0);
    pfr = project(pfr);
    // FIXME avoid branching might help perf!
    if (inBounds(pfr)) {
        c += mix(vec4(0.0), getFromColor(pfr), reflection * mix(1.0, 0.0, pfr.y));
    }
    pto = project(pto);
    if (inBounds(pto)) {
        c += mix(vec4(0.0), getToColor(pto), reflection * mix(1.0, 0.0, pto.y));
    }
    return c;
}

// p : the position
// persp : the perspective in [ 0, 1 ]
// center : the xcenter in [0, 1] \ 0.5 excluded
highp vec2 xskew (highp vec2 p, highp float persp, highp float center) {
    highp float x = mix(p.x, 1.0-p.x, center);
    return (
            (
             vec2( x, (p.y - 0.5*(1.0-persp) * x) / (1.0+(persp-1.0)*x) )
             - vec2(0.5-distance(center, 0.5), 0.0)
             )
            * vec2(0.5 / distance(center, 0.5) * (center<0.5 ? 1.0 : -1.0), 1.0)
            + vec2(center<0.5 ? 0.0 : 1.0, 0.0)
            );
}

highp vec4 transition(highp vec2 op) {
    highp float uz = unzoom * 2.0*(0.5-distance(0.5, progress));
    highp vec2 p = -uz*0.5+(1.0+uz) * op;
    highp vec2 fromP = xskew(
                       (p - vec2(progress, 0.0)) / vec2(1.0-progress, 1.0),
                       1.0-mix(progress, 0.0, persp),
                       0.0
                       );
    highp vec2 toP = xskew(
                     p / vec2(progress, 1.0),
                     mix(pow(progress, 2.0), 1.0, persp),
                     1.0
                     );
    // FIXME avoid branching might help perf!
    if (inBounds(fromP)) {
        return getFromColor(fromP);
    }
    else if (inBounds(toP)) {
        return getToColor(toP);
    }
    return bgColor(op, fromP, toP);
}

void main()
{
    gl_FragColor = transition(R_VEC2(textureCoordinate));
}
