// Author: Hezhk3

// see https://gl-transitions.com/editor/doorway

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

#define progress fract(iTime)
//#define getFromColor(coord) texture2D(inputImageTexture, coord)
//#define getToColor(coord) texture2D(inputImageTexture2, coord)
#define ratio (iResolution.x / iResolution.y)

#define R_VEC2(p) vec2(p.x, 1.0 - p.y)
#define getFromColor(coord) texture2D(inputImageTexture, R_VEC2(coord))
#define getToColor(coord) texture2D(inputImageTexture2, R_VEC2(coord))

// author: gre
// License: MIT
const highp float reflection = 0.4; // = 0.4
const highp float perspective = 0.4; // = 0.4
const highp float depth = 3.0; // = 3

const highp vec4 black = vec4(0.0, 0.0, 0.0, 1.0);
const highp vec2 boundMin = vec2(0.0, 0.0);
const highp vec2 boundMax = vec2(1.0, 1.0);

bool inBounds (highp vec2 p) {
    return all(lessThan(boundMin, p)) && all(lessThan(p, boundMax));
}

highp vec2 project (highp vec2 p) {
    return p * vec2(1.0, -1.2) + vec2(0.0, -0.02);
}

highp vec4 bgColor (highp vec2 p, highp vec2 pto) {
    highp vec4 c = black;
    pto = project(pto);
    if (inBounds(pto)) {
        c += mix(black, getToColor(pto), reflection * mix(1.0, 0.0, pto.y));
    }
    return c;
}


highp vec4 transition (highp vec2 p) {
    highp vec2 pfr = vec2(-1.), pto = vec2(-1.);
    highp float middleSlit = 2.0 * abs(p.x-0.5) - progress;
    if (middleSlit > 0.0) {
        pfr = p + (p.x > 0.5 ? -1.0 : 1.0) * vec2(0.5*progress, 0.0);
        highp float d = 1.0/(1.0+perspective*progress*(1.0-middleSlit));
        pfr.y -= d/2.;
        pfr.y *= d;
        pfr.y += d/2.;
    }
    highp float size = mix(1.0, depth, 1.-progress);
    pto = (p + vec2(-0.5, -0.5)) * vec2(size, size) + vec2(0.5, 0.5);
    if (inBounds(pfr)) {
        return getFromColor(pfr);
    }
    else if (inBounds(pto)) {
        return getToColor(pto);
    }
    else {
        return bgColor(p, pto);
    }
}

void main()
{
    gl_FragColor = transition(R_VEC2(textureCoordinate));
}
