// Author: Hezhk3

// see https://gl-transitions.com/editor/GridFlip

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

#define progress fract(iTime)
#define getFromColor(coord) texture2D(inputImageTexture, coord)
#define getToColor(coord) texture2D(inputImageTexture2, coord)
#define ratio (iResolution.x / iResolution.y)

// License: MIT
// Author: TimDonselaar
// ported by gre from https://gist.github.com/TimDonselaar/9bcd1c4b5934ba60087bdb55c2ea92e5

const highp ivec2 size = ivec2(4);
const highp float pause = 0.1;
const highp float dividerWidth = 0.05;
const highp vec4 bgcolor = vec4(0.0, 0.0, 0.0, 1.0);
const highp float randomness = 0.1;

highp float rand (highp vec2 co) {
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

highp float getDelta(highp vec2 p) {
    highp vec2 rectanglePos = floor(vec2(size) * p);
    highp vec2 rectangleSize = vec2(1.0 / vec2(size).x, 1.0 / vec2(size).y);
    highp float top = rectangleSize.y * (rectanglePos.y + 1.0);
    highp float bottom = rectangleSize.y * rectanglePos.y;
    highp float left = rectangleSize.x * rectanglePos.x;
    highp float right = rectangleSize.x * (rectanglePos.x + 1.0);
    highp float minX = min(abs(p.x - left), abs(p.x - right));
    highp float minY = min(abs(p.y - top), abs(p.y - bottom));
    return min(minX, minY);
}

highp float getDividerSize() {
    highp vec2 rectangleSize = vec2(1.0 / vec2(size).x, 1.0 / vec2(size).y);
    return min(rectangleSize.x, rectangleSize.y) * dividerWidth;
}

highp vec4 transition(highp vec2 p) {
    if(progress < pause) {
        highp float currentProg = progress / pause;
        highp float a = 1.0;
        if(getDelta(p) < getDividerSize()) {
            a = 1.0 - currentProg;
        }
        return mix(bgcolor, getFromColor(p), a);
    }
    else if(progress < 1.0 - pause){
        if(getDelta(p) < getDividerSize()) {
            return bgcolor;
        } else {
            highp float currentProg = (progress - pause) / (1.0 - pause * 2.0);
            highp vec2 q = p;
            highp vec2 rectanglePos = floor(vec2(size) * q);
            
            highp float r = rand(rectanglePos) - randomness;
            highp float cp = smoothstep(0.0, 1.0 - r, currentProg);
            
            highp float rectangleSize = 1.0 / vec2(size).x;
            highp float delta = rectanglePos.x * rectangleSize;
            highp float offset = rectangleSize / 2.0 + delta;
            
            p.x = (p.x - offset)/abs(cp - 0.5)*0.5 + offset;
            highp vec4 a = getFromColor(p);
            highp vec4 b = getToColor(p);
            
            highp float s = step(abs(vec2(size).x * (q.x - delta) - 0.5), abs(cp - 0.5));
            return mix(bgcolor, mix(b, a, step(cp, 0.5)), s);
        }
    }
    else {
        highp float currentProg = (progress - 1.0 + pause) / pause;
        highp float a = 1.0;
        if(getDelta(p) < getDividerSize()) {
            a = currentProg;
        }
        return mix(bgcolor, getToColor(p), a);
    }
}

void main()
{
    gl_FragColor = transition(textureCoordinate);
}
