// Author: Hezhk3

// see https://gl-transitions.com/editor/rotate_scale_fade

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

#define progress fract(iTime)
#define getFromColor(coord) texture2D(inputImageTexture, coord)
#define getToColor(coord) texture2D(inputImageTexture2, coord)
#define ratio (iResolution.x / iResolution.y)

#define PI 3.14159265359

const highp vec2 center = vec2(0.5, 0.5); // = vec2(0.5, 0.5);
const highp float rotations = 1.0; // = 1;
const highp float scale = 8.0; // = 8;
const highp vec4 backColor = vec4(0.15, 0.15, 0.15, 1.0); // = vec4(0.15, 0.15, 0.15, 1.0);

highp vec4 transition (highp vec2 uv) {
    
    highp vec2 difference = uv - center;
    highp vec2 dir = normalize(difference);
    highp float dist = length(difference);
    
    highp float angle = 2.0 * PI * rotations * progress;
    
    highp float c = cos(angle);
    highp float s = sin(angle);
    
    highp float currentScale = mix(scale, 1.0, 2.0 * abs(progress - 0.5));
    
    highp vec2 rotatedDir = vec2(dir.x  * c - dir.y * s, dir.x * s + dir.y * c);
    highp vec2 rotatedUv = center + rotatedDir * dist / currentScale;
    
    if (rotatedUv.x < 0.0 || rotatedUv.x > 1.0 ||
        rotatedUv.y < 0.0 || rotatedUv.y > 1.0)
        return backColor;
    
    return mix(getFromColor(rotatedUv), getToColor(rotatedUv), progress);
}

void main()
{
    gl_FragColor = transition(textureCoordinate);
}
