// Author: Hezhk3

// see https://gl-transitions.com/editor/Swirl

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

#define progress fract(iTime)
#define getFromColor(coord) texture2D(inputImageTexture, coord)
#define getToColor(coord) texture2D(inputImageTexture2, coord)
#define ratio (iResolution.x / iResolution.y)

// License: MIT
// Author: Sergey Kosarevsky
// ( http://www.linderdaum.com )
// ported by gre from https://gist.github.com/corporateshark/cacfedb8cca0f5ce3f7c

const highp float Radius = 1.0;

highp vec4 transition(highp vec2 UV)
{
    highp float T = progress;
    
    UV -= vec2( 0.5, 0.5 );
    
    highp float Dist = length(UV); // distance to center
    
    if ( Dist < Radius )
    {
        highp float Percent = (Radius - Dist) / Radius;
        highp float A = ( T <= 0.5 ) ? mix( 0.0, 1.0, T/0.5 ) : mix( 1.0, 0.0, (T-0.5)/0.5 );
        highp float Theta = Percent * Percent * A * 8.0 * 3.14159;
        highp float S = sin( Theta );
        highp float C = cos( Theta );
        UV = vec2( dot(UV, vec2(C, -S)), dot(UV, vec2(S, C)) );
    }
    UV += vec2( 0.5, 0.5 );
    
    highp vec4 C0 = getFromColor(UV);
    highp vec4 C1 = getToColor(UV);
    
    return mix( C0, C1, T );
}

void main()
{
    gl_FragColor = transition(textureCoordinate);
}
