// Author: Hezhk3

// see https://gl-transitions.com/editor/undulatingBurnOut

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

#define progress fract(iTime)
#define getFromColor(coord) texture2D(inputImageTexture, coord)
#define getToColor(coord) texture2D(inputImageTexture2, coord)
//#define ratio (iResolution.x / iResolution.y)

// License: MIT
// Author: pthrasher
// adapted by gre from https://gist.github.com/pthrasher/8e6226b215548ba12734

const highp float smoothness = 0.03; // = 0.03
const highp vec2 center = vec2(0.5); // = vec2(0.5)
const highp vec3 color = vec3(0.0); // = vec3(0.0)

const highp float M_PI = 3.14159265358979323846;

highp float quadraticInOut(highp float t) {
    highp float p = 2.0 * t * t;
    return t < 0.5 ? p : -p + (4.0 * t) - 1.0;
}

highp float getGradient(highp float r, highp float dist) {
    highp float d = r - dist;
    return mix(
               smoothstep(-smoothness, 0.0, r - dist * (1.0 + smoothness)),
               -1.0 - step(0.005, d),
               step(-0.005, d) * step(d, 0.01)
               );
}

highp float getWave(highp vec2 p){
    highp vec2 _p = p - center; // offset from center
    highp float rads = atan(_p.y, _p.x);
    highp float degs = degrees(rads) + 180.0;
    highp vec2 range = vec2(0.0, M_PI * 30.0);
    highp vec2 domain = vec2(0.0, 360.0);
    highp float ratio = (M_PI * 30.0) / 360.0;
    degs = degs * ratio;
    highp float x = progress;
    highp float magnitude = mix(0.02, 0.09, smoothstep(0.0, 1.0, x));
    highp float offset = mix(40.0, 30.0, smoothstep(0.0, 1.0, x));
    highp float ease_degs = quadraticInOut(sin(degs));
    highp float deg_wave_pos = (ease_degs * magnitude) * sin(x * offset);
    return x + deg_wave_pos;
}

highp vec4 transition(highp vec2 p) {
    highp float dist = distance(center, p);
    highp float m = getGradient(getWave(p), dist);
    highp vec4 cfrom = getFromColor(p);
    highp vec4 cto = getToColor(p);
    return mix(mix(cfrom, cto, m), mix(cfrom, vec4(color, 1.0), 0.75), step(m, -2.0));
}


void main()
{
    gl_FragColor = transition(textureCoordinate);
}
