// Author: Hezhk3

// modified zoom blur from http://transitions.glsl.io/transition/b86b90161503a0023231
// see https://www.shadertoy.com/view/Ml3XR2

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

const highp float strength = 0.3;
const highp float PI = 3.141592653589793;

highp float Linear_ease(highp float begin, highp float change, highp float duration, highp float time) {
    return change * time / duration + begin;
}

highp float Exponential_easeInOut(highp float begin, highp float change, highp float duration, highp float time) {
    if (time == 0.0)
        return begin;
    else if (time == duration)
        return begin + change;
    time = time / (duration / 2.0);
    if (time < 1.0)
        return change / 2.0 * pow(2.0, 10.0 * (time - 1.0)) + begin;
    return change / 2.0 * (-pow(2.0, -10.0 * (time - 1.0)) + 2.0) + begin;
}

highp float Sinusoidal_easeInOut(highp float begin, highp float change, highp float duration, highp float time) {
    return -change / 2.0 * (cos(PI * time / duration) - 1.0) + begin;
}

highp float random(highp vec3 scale, highp float seed) {
    return fract(sin(dot(gl_FragCoord.xyz + seed, scale)) * 43758.5453 + seed);
}

highp vec3 crossFade(highp vec2 uv, highp float dissolve) {
    return mix(texture2D(inputImageTexture, uv).rgb, texture2D(inputImageTexture2, uv).rgb, dissolve);
}

void main() {
    highp vec2 texCoord = textureCoordinate;
    // float progress = sin(iTime*0.5) * 0.5 + 0.5;
    highp float progress = fract(iTime);
    // Linear interpolate center across center half of the image
    highp vec2 center = vec2(Linear_ease(0.5, 0.0, 1.0, progress),0.5);
    highp float dissolve = Exponential_easeInOut(0.0, 1.0, 1.0, progress);
    
    // Mirrored sinusoidal loop. 0->strength then strength->0
    highp float strength = Sinusoidal_easeInOut(0.0, strength, 0.5, progress);
    
    highp vec3 color = vec3(0.0);
    highp float total = 0.0;
    highp vec2 toCenter = center - texCoord;
    
    /* randomize the lookup values to hide the fixed number of samples */
    highp float offset = random(vec3(12.9898, 78.233, 151.7182), 0.0)*0.5;
    
    for (highp float t = 0.0; t <= 20.0; t++) {
        highp float percent = (t + offset) / 20.0;
        highp float weight = 1.0 * (percent - percent * percent);
        color += crossFade(texCoord + toCenter * percent * strength, dissolve) * weight;
        total += weight;
    }
    
    gl_FragColor = vec4(color / total, 1.0);
}
