precision mediump float;

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform lowp float brightness;
uniform lowp float contrast;
uniform highp float exposure;
uniform lowp float shadows;
uniform lowp float highlights;
const mediump vec3 luminanceWeighting = vec3(0.3, 0.3, 0.3);
uniform mediump float hueAdjust;
const highp vec4 kRGBToYPrime = vec4 (0.299, 0.587, 0.114, 0.0);
const highp vec4 kRGBToI = vec4 (0.595716, -0.274453, -0.321263, 0.0);
const highp vec4 kRGBToQ = vec4 (0.211456, -0.522591, 0.31135, 0.0);
const highp vec4 kYIQToR = vec4 (1.0, 0.9563, 0.6210, 0.0);
const highp vec4 kYIQToG = vec4 (1.0, -0.2721, -0.6474, 0.0);
const highp vec4 kYIQToB = vec4 (1.0, -1.1070, 1.7046, 0.0);
uniform lowp float saturation;
const mediump vec3 luminanceWeighting2 = vec3(0.2125, 0.7154, 0.0721);
uniform lowp float temperature;
uniform lowp float tint;
const lowp vec3 warmFilter = vec3(0.93, 0.54, 0.0);
const mediump mat3 RGBtoYIQ = mat3(0.299, 0.587, 0.114, 0.596, -0.274, -0.322, 0.212, -0.523, 0.311);
const mediump mat3 YIQtoRGB = mat3(1.0, 0.956, 0.621, 1.0, -0.272, -0.647, 1.0, -1.105, 1.702);
varying highp vec2 leftTextureCoordinate;
varying highp vec2 rightTextureCoordinate;
varying highp vec2 topTextureCoordinate;
varying highp vec2 bottomTextureCoordinate;
varying highp float centerMultiplier;
varying highp float edgeMultiplier;
uniform lowp vec2 vignetteCenter;
uniform lowp vec3 vignetteColor;
uniform highp float vignetteStart;
uniform highp float vignetteEnd;


void main()
{
    lowp vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);
    // brightness
    textureColor = vec4((textureColor.rgb + vec3(brightness)), textureColor.w);
    textureColor.r = clamp(textureColor.r, 0.0, 1.0);
    textureColor.g = clamp(textureColor.g, 0.0, 1.0);
    textureColor.b = clamp(textureColor.b, 0.0, 1.0);
    textureColor.a = clamp(textureColor.a, 0.0, 1.0);
    // contrast
    textureColor = vec4(((textureColor.rgb - vec3(0.5)) * contrast + vec3(0.5)), textureColor.w);
    textureColor.r = clamp(textureColor.r, 0.0, 1.0);
    textureColor.g = clamp(textureColor.g, 0.0, 1.0);
    textureColor.b = clamp(textureColor.b, 0.0, 1.0);
    textureColor.a = clamp(textureColor.a, 0.0, 1.0);
    // exposure
    textureColor = vec4(textureColor.rgb * pow(2.0, exposure), textureColor.w);
    textureColor.r = clamp(textureColor.r, 0.0, 1.0);
    textureColor.g = clamp(textureColor.g, 0.0, 1.0);
    textureColor.b = clamp(textureColor.b, 0.0, 1.0);
    textureColor.a = clamp(textureColor.a, 0.0, 1.0);
    // highlight shadow
    mediump float luminance = dot(textureColor.rgb, luminanceWeighting);
    mediump float shadow = clamp((pow(luminance, 1.0/(shadows+1.0)) + (-0.76)*pow(luminance, 2.0/(shadows+1.0))) - luminance, 0.0, 1.0);
    mediump float highlight = clamp((1.0 - (pow(1.0-luminance, 1.0/(2.0-highlights)) + (-0.8)*pow(1.0-luminance, 2.0/(2.0-highlights)))) - luminance, -1.0, 0.0);
    lowp vec3 result = vec3(0.0, 0.0, 0.0) + ((luminance + shadow + highlight) - 0.0) * ((textureColor.rgb - vec3(0.0, 0.0, 0.0))/(luminance - 0.0));
    textureColor = vec4(result.rgb, textureColor.a);
    // hue
    highp float YPrime = dot (textureColor, kRGBToYPrime);
    highp float I = dot (textureColor, kRGBToI);
    highp float Q = dot (textureColor, kRGBToQ);
    highp float hue = atan (Q, I);
    highp float chroma = sqrt (I * I + Q * Q);
    hue += (-hueAdjust);
    Q = chroma * sin (hue);
    I = chroma * cos (hue);
    highp vec4 yIQ = vec4 (YPrime, I, Q, 0.0);
    textureColor.r = dot (yIQ, kYIQToR);
    textureColor.g = dot (yIQ, kYIQToG);
    textureColor.b = dot (yIQ, kYIQToB);
    // saturation
    luminance = dot(textureColor.rgb, luminanceWeighting2);
    lowp vec3 greyScaleColor = vec3(luminance);
    textureColor = vec4(mix(greyScaleColor, textureColor.rgb, saturation), textureColor.w);
    // whitebalance
    mediump vec3 yiq = RGBtoYIQ * textureColor.rgb;
    yiq.b = clamp(yiq.b + tint*0.5226*0.1, -0.5226, 0.5226);
    lowp vec3 rgb = YIQtoRGB * yiq;
    lowp vec3 processed = vec3(
    (rgb.r < 0.5 ? (2.0 * rgb.r * warmFilter.r) : (1.0 - 2.0 * (1.0 - rgb.r) * (1.0 - warmFilter.r))), //adjusting temperature
    (rgb.g < 0.5 ? (2.0 * rgb.g * warmFilter.g) : (1.0 - 2.0 * (1.0 - rgb.g) * (1.0 - warmFilter.g))),
    (rgb.b < 0.5 ? (2.0 * rgb.b * warmFilter.b) : (1.0 - 2.0 * (1.0 - rgb.b) * (1.0 - warmFilter.b))));
    textureColor = vec4(mix(rgb, processed, temperature), textureColor.a);
    // sharpen
    mediump vec3 leftTextureColor = texture2D(inputImageTexture, leftTextureCoordinate).rgb;
    mediump vec3 rightTextureColor = texture2D(inputImageTexture, rightTextureCoordinate).rgb;
    mediump vec3 topTextureColor = texture2D(inputImageTexture, topTextureCoordinate).rgb;
    mediump vec4 bottomTextureColor = texture2D(inputImageTexture, bottomTextureCoordinate);
    textureColor = vec4((textureColor.rgb * centerMultiplier - (leftTextureColor * edgeMultiplier + rightTextureColor * edgeMultiplier + topTextureColor * edgeMultiplier + bottomTextureColor.rgb * edgeMultiplier)), bottomTextureColor.w);
    // vignette
    rgb = textureColor.rgb;
    lowp float d = distance(textureCoordinate, vec2(vignetteCenter.x, vignetteCenter.y));
    lowp float percent = smoothstep(vignetteStart, vignetteEnd, d);
    gl_FragColor = vec4(mix(rgb.x, vignetteColor.x, percent), mix(rgb.y, vignetteColor.y, percent), mix(rgb.z, vignetteColor.z, percent), 1.0);
}